@@ -1,4 +1,4 @@
-# $Id: Tabs.pm,v 1.25 2002/11/23 09:52:48 koos Exp $
+# $Id: Tabs.pm,v 1.27 2002/12/02 20:19:37 koos Exp $
 
 package CGI::Widget::Tabs;
 
@@ -10,7 +10,7 @@ use CGI::Widget::Tabs::Heading;
 use URI::Escape();
 use HTML::Entities();
 
-our $VERSION = '1.04';
+our $VERSION = '1.05';
 
 
 
@@ -42,29 +42,15 @@ sub headings {
 
 
 
-
-# ----------------------------------------------
-sub _add_oo_heading {
-# ----------------------------------------------
-    my $self = shift;
-    if ( @_ ) {
-        push @{ $self->{headings} }, shift;
-    }
-    return @{ $self->{headings} || [] };
-}
-
-
-
-
 # ----------------------------------------------
 sub heading {
 # ----------------------------------------------
     #
-    # Create and return a new heading object
+    # Create, add, and return a new heading object
     #
     my $self = shift;
-    my $heading = CGI::Widget::Tabs::Heading->new();
-    $self->_add_oo_heading($heading);
+    my $heading = CGI::Widget::Tabs::Heading->new;
+    push @{ $self->{headings} }, $heading;
     return $heading;
 }
 
@@ -74,7 +60,7 @@ sub heading {
 sub default {
 # ----------------------------------------------
     #
-    # The default active tab
+    # The default active heading
     #
     my $self = shift;
     if ( @_ ) {
@@ -101,7 +87,6 @@ sub cgi_object {
         }
         $self->{cgi_object} = $cgi;
     }
-
     return $self->{cgi_object};
 }
 
@@ -134,7 +119,7 @@ sub active {
     #
     my $self = shift;
 
-    # 1. Tab clicked
+    # 1. Heading clicked
     my $clicked = $self->cgi_object->param($self->cgi_param);
     return $clicked if defined $clicked;  # that was easy :-)
 
@@ -170,6 +155,45 @@ sub class {
 
 
 
+# ----------------------------------------------
+sub wrap {
+# ----------------------------------------------
+    #
+    # wrap to next row after this num of headings
+    #
+    my $self = shift;
+    if ( @_ ) {
+        $self->{wrap} = shift;
+    }
+    return $self->{wrap};
+}
+
+
+
+# ----------------------------------------------
+sub indent {
+# ----------------------------------------------
+    #
+    # Indentation after wrapping to next line?
+    # Evaluates to 1 or 0;
+    #
+    my $self = shift;
+    my $arg = shift;
+    my $indent = 1;  # defaults to 1
+
+    if ( defined $arg ) {
+        # set
+        $indent = 0 if !$arg;
+    } else {
+        # get
+        $indent = 0 if defined $self->{indent} && !$self->{indent};
+    }
+    $self->{indent} = $indent;
+    return $indent;
+}
+
+
+
 # ----------------------------------------------
 sub display {
 # ----------------------------------------------
@@ -194,22 +218,31 @@ sub render {
     my $class       = $self->class;
     my $cgi_param   = $self->cgi_param;
     my $active      = $self->active;
-    my $spacer      = '<td class="'.$class.'_spc"><img height="1" width="1"></td>';
+    my $wrap        = $self->wrap;
+    my $indent      = $self->indent;
+    my $spacer      = '<td class="'.$class.'_spc"><img height="1" width="1"></td>';  # dummy image
+    my $indentation = '<td class="'.$class.'_ind"><img height="1" width="1"></td>'."\n";  # dummy image
     my @html;
     my $all_query_params_but_one = _all_query_params_but_one($cgi, $cgi_param);
     my $url;
 
-    @html = ();
-    push @html, "<!-- Generated by CGI::Widget::Tabs v$VERSION -->\n";
-    push @html, '<table class="',$class,'">',"\n<tr>\n";
-    push @html, $spacer,"\n";
-
     if ( @headings ) {
-        # --- Are we dealing with the OO version of the headings,
-        # --- or is it a simple list (strings or k/v pairs) ?
-        if ( ref($headings[0] ) ) {  # it's a ref -> OO version
+
+        @html = ();
+        push @html, "<!-- Generated by CGI::Widget::Tabs v$VERSION -->\n";
+
+        # --- OO headings
+        if ( ref($headings[0] ) ) {
+            my $heading_nr = 1;  # first heading
+            my $row_nr     = 1;  # first row
             my $param_value;
+
             foreach my $heading ( @headings ) {
+                if ( $heading_nr == 1 ) {   # first one in the row?
+                    push @html, _start_of_row($class, $spacer, $indent, $row_nr, $indentation);
+                }
+
+                # -- actual headings
                 $param_value = $heading->key || $heading->raw_text || $heading->text;
                 push @html, '<td class="',$class;
                 push @html, '_actv' if $param_value eq $active;
@@ -218,14 +251,38 @@ sub render {
                 push @html, &_link( ( $heading->raw_text || HTML::Entities::encode_entities($heading->text) ) , $url );
                 push @html, "</td>";
                 push @html, $spacer,"\n";
+                # -- end of row
+                if ( $wrap && ( $heading_nr == $wrap ) ) {  # last one on the line?
+                    push @html,  "</tr>\n";     # | yes, end this row
+                    push @html,  "</table>\n";  # |
+                    $heading_nr = 0;
+                    $row_nr++;
+                }
+                $heading_nr++;
             }
 
-        } else {  # the first element is a scalar -> ("Tab 1") or ( -t1 => "Tab1" );
-            # --- Did we get the -t=>"Tab" version?
+            # --- all headings printed
+            if ( $heading_nr != 1 )  {     # | Did we just get wrapped to
+                push @html, "</tr>\n";     # | the next line? In that case
+                push @html, "</table>\n";  # | we don't need an end-of-row.
+            }
+        }
+
+        # --- simple list headings
+        else {  # the first element is a scalar -> ("Heading") or ( -h => "Heading" );
+            # --- Did we get the -h => "Heading" version?
             if ( substr($headings[0],0,1) eq '-' ) {
+                my $heading_nr = 1;  # first heading
+                my $row_nr     = 1;  # first row
                 my $heading_key;
                 my $heading_text;
                 while ( @headings ) {
+
+                    if ( $heading_nr == 1 ) {   # first one in the row?
+                        push @html, _start_of_row($class, $spacer, $indent, $row_nr, $indentation);
+                    }
+
+                    # -- actual headings
                     $heading_key  = shift @headings;
                     $heading_text = shift @headings;
                     push @html, '<td class="',$class;
@@ -234,28 +291,82 @@ sub render {
                     push @html, &_link($heading_text,'?'.$all_query_params_but_one.$cgi_param.'='.URI::Escape::uri_escape($heading_key));
                     push @html, "</td>";
                     push @html, $spacer,"\n";
+
+                    # -- end of row
+                    if ( $wrap && ( $heading_nr == $wrap ) ) {  # last one on this row?
+                        push @html,  "</tr>\n";     # | yes, end this row
+                        push @html,  "</table>\n";  # |
+                        $heading_nr = 0;
+                        $row_nr++;
+                    }
+                    $heading_nr++;
                 }
+
+                # --- all headings printed
+                if ( $heading_nr != 1 )  {     # | Did we just get wrapped to
+                    push @html, "</tr>\n";     # | the next row? In that case
+                    push @html, "</table>\n";  # | we don't need an end-of-row.
+                }
+
             } else {
-                # --- No, we got the ("Tab1", "Tab2", ...)  version.
+                # --- No, we got the ("Heading1", "Heading2", ...)  version.
+                my $heading_nr = 1;  # first heading
+                my $row_nr     = 1;  # first row
                 foreach my $heading ( @headings ) {
+
+                    if ( $heading_nr == 1 ) {   # first one in the row?
+                        push @html, _start_of_row( $class, $spacer, $indent, $row_nr, $indentation );
+                    }
+
+                    # -- actual headings
                     push @html, '<td class="',$class;
                     push @html, '_actv' if $heading eq $active;
                     push @html, '">';
                     push @html, &_link($heading,'?'.$all_query_params_but_one.$cgi_param.'='.URI::Escape::uri_escape($heading));
                     push @html, "</td>";
                     push @html, $spacer,"\n";
+
+                    # -- end of row
+                    if ( $wrap && ( $heading_nr == $wrap ) ) {  # last one on the row?
+                        push @html,  "</tr>\n";     # | yes, end this row
+                        push @html,  "</table>\n";  # |
+                        $heading_nr = 0;
+                        $row_nr++;
+                    }
+                    $heading_nr++;
+                }
+
+                # --- all headings printed
+                if ( $heading_nr != 1 )  {     # | Did we just get wrapped to
+                    push @html, "</tr>\n";     # | the next row? In that case
+                    push @html, "</table>\n";  # | we don't need an end-of-row.
                 }
             }
         }
     }
 
-    push @html, "</tr>\n</table>\n";
     push @html, "<!-- End CGI::Widget::Tabs v$VERSION -->\n";
     return join("",@html);
 }
 
 
 
+# ----------------------------------------------
+sub _start_of_row {
+# ----------------------------------------------
+    my ( $class, $spacer, $indent, $row_nr, $indentation ) = @_;
+    my @html = ();
+    push @html, "<table class=\"",$class,'">',"\n";
+    push @html,  "<tr>\n";
+    if ( $indent && $row_nr > 1 ) {                     # = print indents if
+        push @html, ( $indentation x ($row_nr - 1));    # = necessary
+    }                                                   # =
+    push @html, $spacer,"\n";                           # each row starts with a spacer
+    return @html;
+}
+
+
+
 # ----------------------------------------------
 sub _link {
 # ----------------------------------------------
@@ -282,8 +393,8 @@ sub _all_query_params_but_one {
     return $all_query_params_but_one;
 }
 
-
 1;
+
 __END__
 
 =head1 NAME
@@ -304,12 +415,15 @@ CGI::Widget::Tabs - Create tab widgets in HTML
     $tab->class("my_tab");       # the CSS class to use for markup
     $tab->cgi_object($cgi);      # the object holding the query params
     $tab->cgi_param("t");        # the CGI query parameter to use
+    $tab->wrap(4);               # wrap after 4 headings...
+    $tab->indent(1);             # ...and add indentation
     $tab->render;                # the resulting HTML code
     $tab->display;               # same as `print $tab->render'
 
+
     $h = $tab->heading;               # new OO heading for this tab
     $h->text("TV Listings");          # heading text. HTML escaped
-    $h->raw_text("TV&nbsp;Listings"); # similar. Not HTML escaped
+    $h->raw_text("TV&nbsp;Listings"); # similar. But passed as is
     $h->key("tv");                    # key identifying this heading
     $h->url("whatsontonight.com");    # redirect URL for this heading
 
@@ -327,20 +441,27 @@ reasons for taking this approach:
 1. For the end user not to be directed to YAL or YAP (yet another link / yet
 another page), but keep it all together: The single point of entry paradigm.
 
-2. For the Perl hacker to generate and display multiple data sources within
-the same script environment.
+2. As a consequence the end user deals with a more consistent and integrated
+GUI. This will give a better "situational awareness" within the application.
+
+3. For the Perl hacker to handle multiple related data sources within the
+same script environment.
 
 
 As an example the following tabs could be used on a web page for someone's
 spotting hobby:
 
-      ________      ________      _____________
-     / Planes \    / Trains \    / Automobiles \
------------------------------------------------------
+      __________      __________      __________
+     /  Planes  \    /  Trains  \    / Classics \
+------------------------------------------------------
+         _________
+        /  Bikes  \
+------------------------
 
-The nice thing about CGI::Widget::Tabs is that the tabs know their internal
-state. So you can ask a tab for instance which tab heading has been clicked
-by the user. This way you get instant feedback.
+As you can see, the headings wrap at three and a small indentation is added
+to the start of the next row. The nice thing about CGI::Widget::Tabs is that
+the tabs know their internal state. So you can ask a tab for instance which
+heading has been clicked by the user. This way you get instant feedback.
 
 =head2 "Hey Gorgeous!"
 
@@ -355,42 +476,52 @@ CGI::Widget::Tabs use that. See the class() method for how to do this.
 
 Tab headings are the things that identify a tab page. Observe the spotting
 example above. Here the different tab pages are identified by the strings
-"Planes", "Trains", and "Automobiles". These tab headings come in two
-flavors: simple headings and object oriented (OO) headings. Simple headings
-are the most easy and convenient ones to use. For instance in the spotting
-example the three tabs headings "Planes", "Trains", and "Automobiles" are
+"Planes", "Trains", "Classics" and "Bikes". They form the heading of each
+page. These tab headings come in two flavors: simple headings and object
+oriented (OO) headings. Simple headings are the most easy and convenient ones
+to use. For instance in the spotting example the four tabs headings are
 easily created by feeding these words as a list to a CGI::Widget::Tabs
-object. And then you are almost done: the headings can be displayed, each
-heading gets it's own self referencing URL and the tab object knows if you
-clicked either one of them. The corresponding statement would be:
+object. And then you are almost done: the headings can be displayed and each
+heading gets it's own self referencing URL. The corresponding statements are:
 
-    $tab->headings("Planes", "Trains", "Automobiles");
+    my $tab = CGI::Widget::Tabs->new;
+    $tab->headings( qw/Planes Trains Classics Bikes/ );
+    $tab->wrap(3);
 
-assuming $tab is the CGI::Widget::Tabs object. This fast and easy to use
-mechanism has it's downside nonetheless. For instance the URL is always a
-self referencing URL. Also future extensions --like support for thumbnail
-images-- is almost impossible. To allow for this extensibility headings can
-be defined in OO fashion. The OO statements to produce the headings above
-could be something like:
+This fast and easy to use mechanism has it's downside nonetheless. For
+instance the URL is always a self referencing URL. Also future extensions
+--like support for thumbnail images-- is almost impossible. To allow for this
+extensibility headings can be defined in OO fashion. The OO statements to
+produce the headings would be something like:
 
-    foreach $ht ( qw/Planes Trains Automobiles/ ) {
-        $h = $tab->heading();  # create/add a heading object
+    my $tab = CGI::Widget::Tabs->new;
+    foreach $ht ( qw/Planes Trains Classics Bikes/ ) {
+        $h = $tab->heading();  # create and add a heading object
         $h->text($ht);         # display $ht as heading text
     }
+    $tab->wrap(3);
 
 
 Here the text() method makes the heading object display the text given by
 $ht. Look at the heading() method elsewhere in this document to see which
 other methods are available to define the properties and behaviour of OO
-headings. Note that you can not mix simple headings with OO headings. If you
-already defined simple headings you can't go adding OO headings or vice
-versa. You need to stick with one type.
+headings. Did you see that in both accounts the indent() method was not used?
+That is because indentation is automatic. You get that for free. Actually,
+you must explicitely turn if off if you don't want it! Note that you can not
+mix simple headings with OO headings. If you already defined simple headings
+you can't go adding OO headings or vice versa. You need to stick with one
+type.
 
 
 
 
 =head1 COMMON METHODS FOR TAB OBJECTS
 
+Common methods deal with tab objects in general. They describe or define the
+global widget properties and it's behaviour.
+
+
+
 =over 4
 
 =item B<new()>
@@ -432,9 +563,11 @@ such an object handler must provide a param() method with corresponding
 behaviour as do CGI or CGI::Minimal. Note that currently only CGI and
 CGI::Minimal have been tested. Example:
 
+    # set
     my $cgi = CGI::Minimal->new;
     $tab->cgi_object($cgi);
 
+    # get
     my $cgi = $tab->cgi_object;
 
 
@@ -471,7 +604,7 @@ use "tab" by default causing conflicts. Example:
 
 Sets/retrieves the name of the CSS class used for the tabs markup. If the
 optional argument STRING is given the class is set, otherwise it is
-retrieved. In the accompanying style sheet, there are four class elements you
+retrieved. In the accompanying style sheet, there are five class elements you
 need to provide:
 
 =over 4
@@ -484,23 +617,72 @@ need to provide:
 
 =item 4. A td element for the spacers
 
+=item 5. A td element for the indentation (if needed)
+
 =back
 
-The class names of these elements are gotten from the class() method. The
-element for an active tab gets "_actv" added to the class name. The spacer
-element gets "_spc" added. For instance, if you'd run
+The class names of these elements are directly borrowed from the class()
+method. The td elements for the active tab, the spacers and the indentations
+are suffixed with "_actv", "_spc" and "_ind" respectively. For instance, if
+you'd run
 
     $tab->class("my_tab");
 
-then the four elements look like:
+then the elements look like:
 
     <table class="my_tab">    # the entire table
     <td class="my_tab">       # normal tab
     <td class="my_tab_actv">  # highlighted tab
     <td class="my_tab_spc">   # spacer
+    <td class="my_tab_ind">   # indentation
+
+If you don't wrap headings, then ofcourse you won't need to specify the
+indentation td's. By the way, the indentation will usually look most natural
+if it has the same width as the spacers or a multiple. Look at the example in
+the EXAMPLE section to see how this all works out.
 
 
-Look at the example in the EXAMPLE section to see how this works out.
+
+=item B<wrap(NUMBER)>
+
+Sets or retrieves the wrap setting. Without arguments the current wrap
+setting is returned. If the argument NUMBER is given the headings will wrap
+to the next row after NUMBER headings. By default headings are not wrapped.
+
+
+
+=item B<indent(LVAL)>
+
+Sets/retrieves the indentation setting. Without arguments the current setting
+is returned. indent() specifies if indentation should be added to the
+next row if the headings get wrapped. By default indent() is set to
+TRUE. You must explicitaly switch off indentation for the desired effect.
+indent() is a toggle. The optional argument LVAL can be any argument
+evaluating to a logical value.
+
+The purpose of swithing off indentation is to simulate a vertical menu. In
+the spotting example, running
+
+    $tab->wrap(1);
+    $tab->indent(0);
+
+could result in something like:
+
+      __________
+     |  Planes  |
+    --------------
+      __________
+     |  Trains  |
+    --------------
+      __________
+     | Classics |
+    --------------
+      __________
+     |  Bikes   |
+    --------------
+
+
+You probably need to tweak your style sheet to have it look nicely.
 
 
 
@@ -535,6 +717,11 @@ document to see how you can influence the markup of the tab widget. Example:
 
 =head1 METHODS FOR SIMPLE HEADINGS
 
+These methods define the properties of simple headings. As the simple
+headings are indeed very simple the wording "headings" doesn't fit. There is
+actually only one method :-)
+
+
 =over 4
 
 =item B<headings(LIST)>
@@ -553,18 +740,18 @@ headings are set. You can specify LIST in two ways:
 
 The keyword/value list comes in handy if you don't want to check the value
 returned by active() against very long words. Moreover, if you change the tab
-headings (upper/lower case!) but use the same keys you don't need to change
-your code. So it is less  error prone. As a pleasant side effect, the URL's
-get significantly shorter. Do notice that the keys want to be unique.
+headings (upper/lower case, typo's) but use the same keys you don't need to
+change your code. So it is less  error prone. As a pleasant side effect, the
+URL's get significantly shorter. Do notice that the keys want to be unique.
 Example:
 
     # plain list
-    $tab->headings( qw/Planes Trains Automobiles/ );
+    $tab->headings( qw/Hardware Software Mobiles/ );
 
     # k/v list
-    $tab->headings( -p => "Planes",
-                    -t => "Trains",
-                    -a => "Automobiles" );
+    $tab->headings( -h => "Hardware",
+                    -s => "Software",
+                    -m => "Mobiles" );
 
     # what have we got sofar?
     my @h = $tab->headings;
@@ -583,6 +770,9 @@ That won't work.
 
 =head1 METHODS FOR OO HEADINGS
 
+These methods define the properties and behaviour of the object oriented
+headings. Each OO heading can be tailored to specific requirements.
+
 =over 4
 
 =item B<heading()>
@@ -603,13 +793,13 @@ methods:
 
 Sets/retrieves the heading text for the OO heading. If the optional argument
 STRING is given, the text will be set otherwise it will be retrieved.
-On actual display STRING will be HTML escaped. Example:
+On actual display STRING will be HTML escaped. Examples:
 
     # set heading text
     $h1->text("Names A > L");
     $h2->text("Names M < Z");
 
-    # text of the 4th heading
+    # get the text of the 4th heading
     my $text = ($tab->headings)[3]->text;
 
 
@@ -626,7 +816,8 @@ useful if you have some HTML preformatted text to display. Example:
     $h2->raw_text("Names&nbsp;M&nbsp;&lt;&nbsp;Z");
 
     # text of the 4th heading
-    my $text = ($tab->headings)[3]->text;
+    my $text = ($tab->headings)[3]->raw_text;
+
 
 
 =item B<key(STRING)>
@@ -696,7 +887,7 @@ simple headings. Copy it and run it through the webservers CGI engine. (For a
 even more complete and useful demo with multiple tabs, see the file
 tabs-demo.pl in the CGI::Widget::Tabs installation directory.)
 
-    #!/usr/bin/perl -w
+    # !/usr/bin/perl -w
 
     use CGI::Widget::Tabs;
     use CGI;
@@ -706,10 +897,11 @@ tabs-demo.pl in the CGI::Widget::Tabs installation directory.)
 
     <head>
     <style type="text/css">
-    table.my_tab   { border-bottom: solid thin black }
-    td.my_tab      { padding: 2 12 2 12; background-color: #FAFAD2 }
-    td.my_tab_actv { padding: 2 12 2 12; background-color: #C0D4E6 }
-    td.my_tab_spc  { width: 15 }
+    table.my_tab   { border-bottom: solid thin black; text-align: center }
+    td.my_tab      { padding: 2 12 2 12; width: 80; background-color: #FAFAD2 }
+    td.my_tab_actv { padding: 2 12 2 12; width: 80; background-color: #C0D4E6 }
+    td.my_tab_spc  { width: 5 }
+    td.my_tab_ind  { width: 15 }
     </style></head>
     <body>
     EOT
@@ -718,8 +910,11 @@ tabs-demo.pl in the CGI::Widget::Tabs installation directory.)
     my $tab = CGI::Widget::Tabs->new;
     $tab->cgi_object($cgi);
     $tab->class("my_tab");
-    $tab->headings( "Hardware", "Lease Cars", "Xerox", "Mobiles");
-    $tab->default("Lease Cars");
+    $tab->headings( "Hardware", "Software", "Lease Cars", "Xerox", "Mobiles");
+    $tab->wrap(3);
+    # $tab->wrap(1);    # |uncomment to see the effect of
+    # $tab->indent(0);  # |wrapping at 1 without indentation
+    $tab->default("Software");
     $tab->display;
     print "<br>We now should run some intelligent code ";
     print "to process <strong>", $tab->active, "</strong><br>";
@@ -733,6 +928,12 @@ tabs-demo.pl in the CGI::Widget::Tabs installation directory.)
 As a side effect, the CGI query parameter to identify the tab (see the
 cgi_param() method) is always moved to the end of the query string.
 
+With OO headings text() is completely isolated from raw_text(). They are
+both seperate properties. Consequently the value set with text() can not be
+overwritten with a value set with raw_text() or vice-versa. Neither can you
+retrieve the current heading text unless you B<know> which one of the two you
+used to set the heading text. text() and raw_text() should probably use the
+same container. This will be fixed in a future release.
 
 
 =head1 CONTRIBUTIONS
@@ -745,7 +946,7 @@ release.
 
 
 
-=head1 CREDITS
+=head1 ACKNOWLEDGEMENTS
 
 =over 4
 
